@@ -4,6 +4,7 @@
 #
 # Copyright (C) 2018, James R. Barlow (https://github.com/jbarlow83/)
 
+import logging
 import re
 import sys
 from collections import namedtuple
@@ -16,7 +17,7 @@ from warnings import warn
 from lxml import etree
 from lxml.etree import QName, XMLParser, XMLSyntaxError, parse
 
-from .. import Name, PdfError, Stream, String
+from .. import Name, Stream, String
 from .. import __version__ as pikepdf_version
 
 XMP_NS_DC = "http://purl.org/dc/elements/1.1/"
@@ -64,6 +65,14 @@ XPACKET_END = b"""\n<?xpacket end="w"?>\n"""
 
 XmpContainer = namedtuple('XmpContainer', ['rdf_type', 'py_type', 'insert_fn'])
 
+log = logging.getLogger(__name__)
+
+
+class NeverRaise(Exception):
+    """An exception that is never raised"""
+
+    pass  # pylint: disable=unnecessary-pass
+
 
 class AltList(list):
     pass
@@ -87,7 +96,7 @@ LANG_ALTS = frozenset(
 # These are the illegal characters in XML 1.0. (XML 1.1 is a bit more permissive,
 # but we'll be strict to ensure wider compatibility.)
 re_xml_illegal_chars = re.compile(
-    r"(?u)[^\x09\x0A\x0D\x20-\uD7FF\uE000-\uFFFD\u10000-\u10FFFF]"
+    r"(?u)[^\x09\x0A\x0D\x20-\U0000D7FF\U0000E000-\U0000FFFD\U00010000-\U0010FFFF]"
 )
 re_xml_illegal_bytes = re.compile(
     br"[^\x09\x0A\x0D\x20-\xFF]|&#0;"
@@ -159,6 +168,8 @@ class AuthorConverter:
     def docinfo_from_xmp(xmp_val):
         if isinstance(xmp_val, str):
             return xmp_val
+        elif xmp_val is None or xmp_val == [None]:
+            return None
         else:
             return '; '.join(xmp_val)
 
@@ -249,12 +260,15 @@ class PdfMetadata(MutableMapping):
     NS = {prefix: uri for uri, prefix in DEFAULT_NAMESPACES}
     REVERSE_NS = {uri: prefix for uri, prefix in DEFAULT_NAMESPACES}
 
-    def __init__(self, pdf, pikepdf_mark=True, sync_docinfo=True):
+    def __init__(
+        self, pdf, pikepdf_mark=True, sync_docinfo=True, overwrite_invalid_xml=True
+    ):
         self._pdf = pdf
         self._xmp = None
         self.mark = pikepdf_mark
         self.sync_docinfo = sync_docinfo
         self._updating = False
+        self.overwrite_invalid_xml = overwrite_invalid_xml
 
     def load_from_docinfo(self, docinfo, delete_missing=False, raise_failure=False):
         """Populate the XMP metadata object with DocumentInfo
@@ -302,7 +316,7 @@ class PdfMetadata(MutableMapping):
             msg = (
                 "The metadata field {} with value '{}' has no XMP equivalent, "
                 "so it was discarded"
-            ).format(extra, docinfo.get(extra))
+            ).format(extra, repr(docinfo.get(extra)))
             if raise_failure:
                 raise ValueError(msg)
             else:
@@ -316,39 +330,54 @@ class PdfMetadata(MutableMapping):
         self._load_from(data)
 
     def _load_from(self, data):
-        try:
-            self._xmp = parse(BytesIO(data))
-        except XMLSyntaxError:
-            data = re_xml_illegal_bytes.sub(b'', data)
+        if data.strip() == b'':
+            data = XMP_EMPTY  # on some platforms lxml chokes on empty documents
+
+        def basic_parser(xml):
+            return parse(BytesIO(xml))
+
+        def strip_illegal_bytes_parser(xml):
+            return parse(BytesIO(re_xml_illegal_bytes.sub(b'', xml)))
+
+        def recovery_parser(xml):
+            parser = XMLParser(recover=True)
+            return parse(BytesIO(xml), parser)
+
+        def replace_with_empty_xmp(_xml=None):
+            log.warning("Error occurred parsing XMP, replacing with empty XMP.")
+            return basic_parser(XMP_EMPTY)
+
+        if self.overwrite_invalid_xml:
+            parsers = [
+                basic_parser,
+                strip_illegal_bytes_parser,
+                recovery_parser,
+                replace_with_empty_xmp,
+            ]
+        else:
+            parsers = [basic_parser]
+
+        for parser in parsers:
             try:
-                self._xmp = parse(BytesIO(data))
-            except XMLSyntaxError as e:
+                self._xmp = parser(data)
+            except (XMLSyntaxError if self.overwrite_invalid_xml else NeverRaise) as e:
                 if str(e).startswith("Start tag expected, '<' not found") or str(
                     e
                 ).startswith("Document is empty"):
-                    # This is usually triggered by processing instructions
-                    # in another otherwise empty document, or empty documents,
-                    # which we consider safe to coerce to a well-formed
-                    # XMP. For harder cases like truncated XMP, we want to
-                    # raise the exception so that someone is alerted.
-                    parser = XMLParser(recover=True)
-                    self._xmp = parse(BytesIO(XMP_EMPTY), parser)
-                else:
-                    raise PdfError(
-                        str(e) + data.decode('utf-8', errors='escape')
-                    ) from e
-        pis = self._xmp.xpath('/processing-instruction()')
-        for pi in pis:
-            etree.strip_tags(self._xmp, pi.tag)
-        try:
-            self._get_rdf_root()
-        except ValueError:
-            if self._xmp.find('.', self.NS).tag == '{adobe:ns:meta/}xmpmeta':
-                # Looks like: <x:xmpmeta></x:xmpmeta>, so reload with template
-                # that includes <rdf:RDF>
-                return self._load_from(XMP_EMPTY)
+                    self._xmp = replace_with_empty_xmp()
+                    break
             else:
-                raise  # Probably not XMP
+                break
+
+        try:
+            pis = self._xmp.xpath('/processing-instruction()')
+            for pi in pis:
+                etree.strip_tags(self._xmp, pi.tag)
+            self._get_rdf_root()
+        except (Exception if self.overwrite_invalid_xml else NeverRaise) as e:
+            log.warning("Error occurred parsing XMP", exc_info=e)
+            self._xmp = replace_with_empty_xmp()
+        return
 
     @ensure_loaded
     def __enter__(self):
@@ -369,7 +398,8 @@ class PdfMetadata(MutableMapping):
         The standard mapping is described here:
             https://www.pdfa.org/pdfa-metadata-xmp-rdf-dublin-core/
         """
-        self._pdf.docinfo  # Touch object to ensure it exists
+        # Touch object to ensure it exists
+        self._pdf.docinfo  # pylint: disable=pointless-statement
         for uri, element, docinfo_name, converter in self.DOCINFO_MAPPING:
             qname = QName(uri, element)
             try:
@@ -388,6 +418,12 @@ class PdfMetadata(MutableMapping):
                         )
                     )
                     value = None
+                except Exception as e:
+                    raise ValueError(
+                        "An error occurred while updating DocumentInfo field {} from XMP {} with value {}".format(
+                            docinfo_name, qname, value
+                        )
+                    ) from e
             if value is None:
                 if docinfo_name in self._pdf.docinfo:
                     del self._pdf.docinfo[docinfo_name]
@@ -608,7 +644,7 @@ class PdfMetadata(MutableMapping):
                 node = etree.SubElement(rdfdesc, self._qname(key))
                 add_array(node, val)
             elif isinstance(val, str):
-                rdfdesc = etree.SubElement(
+                _rdfdesc = etree.SubElement(
                     rdf,
                     QName(XMP_NS_RDF, 'Description'),
                     attrib={
